/* This is "bisect3.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and a test driver for the
   bisection method as described in Chapter 9 of the text. */


#include <stdio.h>
#include <math.h>



/* This is x^4 + 2x^3 - 14x^2 + 2x + 1, whose root we seek. */

double f (double x)
{
  return(1 + x*(2 + x*(-14 + x*(2 + x))));
}



/* Uses the bisection method to find an approximate root for "f",
   where f(pos) > 0 and f(neg) < 0.  It narrows down the difference
   between "pos" and "neg" until the absolute value of the difference
   between two successive averages becomes small relative to the first
   of the two averages.  It then returns the final average. */

double bisection (double pos, double neg)
{

  double avg, oldavg;    /* Keeps track of two successive averages */
                         /* of "pos" and "neg". */


  /* Apply the bisection narrowing process until the difference
     between the two successive averages "oldavg" and "avg" becomes
     small relative to "oldavg".  At that point, return "avg" as
     the approximate root of "f". */

  avg = (pos+neg)/2.0;
  
  do {
    oldavg = avg;
    if (f(avg) >= 0) {
      pos = avg;
    }
    else {
      neg = avg;
    }
    avg = (pos + neg) / 2.0;
  }
  while (fabs((oldavg-avg)/oldavg) > 1e-10);
  
  return(avg);

}



/* Prompts for the positive and negative approximations to the root of
   the function "f" defined above, then computes and displays an
   approximation to f's root. */

void main ()
{

  double positive,    /* "Positive" guess, meaning f(positive) > 0 */
         negative,    /* "Negative" guess, meaning f(negative) < 0 */
         root;        /* Approximate root of "f". */


  /* Obtain a valid positive guess. */

  do {
    printf("Enter positive guess: ");
    scanf("%lf", &positive);
  }
  while (f(positive) <= 0);


  /* Obtain a valid negative guess. */

  do {
    printf("Enter negative guess: ");
    scanf("%lf", &negative);
  }
  while (f(negative) >= 0);


  /* Obtain and display an approximate root of "f". */

  root = bisection(positive, negative);
  printf("An approximate root is %.10g\n", root);
  printf("f(approximate root) is %.10g\n", f(root));

}
