/* This is "newton8.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is a correct (albeit unwieldy) implementation and
   test driver for Newton's method with complex numbers as
   described in Chapter 16 of the text. */


#include <stdio.h>
#include <math.h>



/* This structure is designed to represent complex numbers. */

typedef struct {
  double real;      /* Real component of complex number */
  double imag;      /* Imaginary component of complex number */
} complex;



/* This is x^2 - 6x + 13, whose root we seek. */

complex f (complex x)
{
  complex result;
  result.real = x.real*x.real - x.imag*x.imag -
                   6*x.real + 13;
  result.imag = 2*x.real*x.imag - 6*x.imag;
  return(result);
}



/* This is 2x - 6, which is the first derivative of f. */

complex fprime (complex x)
{
  complex result;
  result.real = 2*x.real - 6;
  result.imag = 2*x.imag;
  return(result);
}



/* An implementation of Newton's method that finds complex roots.
   It improves the guess "g0" for the root of "f" until the absolute
   value of the difference between two successive guesses becomes
   small relative to the first of the two guesses.  It then returns
   the improved guess. */

complex newton (complex g0)
{

  complex oldg0,   /* The value of g0 during the previous iteration. */
          fg0,     /* The value of f(g0). */
          fpg0;    /* The value of fprime(g0). */


  /* This implementation is unwieldy because it operates directly
     on the representation of "complex" structures instead of relying
     on an abstract datatype. */

  do {
    oldg0 = g0;
    fg0 = f(g0);
    fpg0 = fprime(g0);

    g0.real = g0.real -
      (fg0.real*fpg0.real + fg0.imag*fpg0.imag) / 
        (fg0.real*fg0.real + fpg0.imag*fpg0.imag);

    g0.imag = g0.imag -
      (fg0.imag*fpg0.real - fg0.real*fpg0.imag) / 
        (fg0.real*fg0.real + fpg0.imag*fpg0.imag);
  }
  while
    (sqrt(pow(1-(oldg0.real*g0.real + oldg0.imag*g0.imag) /
              (g0.real*g0.real + g0.imag*g0.imag), 2) +
          pow((oldg0.imag*g0.real - oldg0.real*g0.imag) /
              (g0.real*g0.real + g0.imag*g0.imag), 2)) >
     1e-10);

  return(g0);

}



/* A test driver that uses "newton" to find a complex root
   of "f" given a guess read from the keyboard. */

void main (void)
{
  complex guess, root;
  printf("Please enter an initial guess: ");
  scanf("%lf%lfi", &guess.real, &guess.imag);
  root = newton(guess);
  printf("An approximate root is %.10g%+.10gi\n",
         root.real, root.imag);
}
