/* This is "rod7.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program solves the rod-stacking problem described in
   Chapter 13 of the text. */


#include <stdio.h>
#include <math.h>



/* This structure is designed to represent a point in a plane. */

typedef struct {
  double x;         /* The x-coordinate of the point */
  double y;         /* The y-coordinate of the point */
} point;



/* Returns the square of "x". */

double sq (double x) {
  return(x*x);
}



/* Given the lengths of three sides of a triangle, returns
   the cosine of the angle between the first two. */

double cosine (double adj1, double adj2, double opp)
{
  return((sq(adj1) + sq(adj2) - sq(opp)) / (2*adj1*adj2));
}



/* Returns the distance between the points "pt1" and "pt2". */

double distance (point pt1, point pt2) {
  return(sqrt(sq(pt1.x-pt2.x) + sq(pt1.y-pt2.y)));
}



/* This function returns the x-y coordinate of the center of a
   circular rod of radius "rTop" that is supported by two
   other rods.  The left supporting rod has radius "rLeft" and
   its center is at "pLeft".  The right supporting rod has
   radius "rRight" and its center is at "pRight". */

point threeRod (point pLeft, point pRight,
                double rLeft, double rRight, double rTop)
{
  double a, b, c, d, e;   /* Sides of the two triangles from */
                          /* Figure 13.3 of the book. */

  double cosA, sinA,      /* Cosine and sine of angle alpha */
                          /* from Figure 13.3 of the book */
      
         cosB, sinB;      /* Cosine and sine of angle beta */
                          /* from Figure 13.3 of the book */

  point pTop;             /* The center coordinate of the */
                          /* supported rod. */


  /* Calculate the lengths of the sides of the triangles. */

  a = rLeft + rTop;
  b = rRight + rTop;
  c = distance(pLeft, pRight);
  d = pRight.x - pLeft.x;
  e = pRight.y - pLeft.y;


  /* Calculate the cosines and sines of alpha and beta. */

  cosA = cosine(a, c, b);
  sinA = sqrt(1 - sq(cosA));
  cosB = d/c;
  sinB = e/c;


  /* Compute and return the center of the supported rod. */

  pTop.x = pLeft.x + a * (cosA*cosB - sinA*sinB);
  pTop.y = pLeft.y + a * (sinA*cosB + sinB*cosA);
  return(pTop);

}


/* Solves the rod-stacking problem described in Chapter 13 of the
   book.  The known center and coordinates are built into this
   function.  It calculates and displays the center coordinates
   of the five supported rods. */

void main (void)
{
  point p1, p2, p3, p4, p5, p6, p7, p8;    /* Center coordinates */
  double r1, r2, r3, r4, r5, r6, r7, r8;   /* Radii */


  /* Set up the known values. */

  p1.x = 3; p1.y = 3; r1 = 3;
  p2.x = 20; p2.y = 4; r2 = 4;
  p3.x = 38; p3.y = 2; r3 = 2;
  r4 = 9; r5 = 8; r6 = 3; r7 = 5; r8 = 4;


  /* Calculate the center coordinates of the supported rods. */

  p4 = threeRod(p1, p2, r1, r2, r4);
  p5 = threeRod(p2, p3, r2, r3, r5);
  p6 = threeRod(p4, p5, r4, r5, r6);
  p7 = threeRod(p6, p5, r6, r5, r7);
  p8 = threeRod(p4, p7, r4, r7, r8);


  /* Display results. */

  printf("Rod    X       Y\n");
  printf(" 4 %7.2f %7.2f\n", p4.x, p4.y);
  printf(" 5 %7.2f %7.2f\n", p5.x, p5.y);
  printf(" 6 %7.2f %7.2f\n", p6.x, p6.y);
  printf(" 7 %7.2f %7.2f\n", p7.x, p7.y);
  printf(" 8 %7.2f %7.2f\n", p8.x, p8.y);

}
